import random
import hashlib
from packaging import version
from instapv.config import Config

_config = Config() 

class DeviceGenerator:
    
    def __init__(self):
        # All Good Devices
        self.DEVICES = [
            '24/7.0; 380dpi; 1080x1920; OnePlus; ONEPLUS A3010; OnePlus3T; qcom',
            '23/6.0.1; 640dpi; 1440x2392; LGE/lge; RS988; h1; h1',
            '24/7.0; 640dpi; 1440x2560; HUAWEI; LON-L29; HWLON; hi3660',
            '23/6.0.1; 640dpi; 1440x2560; ZTE; ZTE A2017U; ailsa_ii; qcom',
            '23/6.0.1; 640dpi; 1440x2560; samsung; SM-G935F; hero2lte; samsungexynos8890',
            '23/6.0.1; 640dpi; 1440x2560; samsung; SM-G930F; herolte; samsungexynos8890',
        ]

        self.REQUIRED_ANDROID_VERSION = '2.2'
        self.USER_AGENT_FORMAT = 'Instagram %s Android (%s/%s; %s; %s; %s; %s; %s; %s; %s)'

        self._userAgent = None

        self._androidVersion = None
        self._androidRelease = None
        self._dpi = None
        self._resolution = None
        self._manufacturer = None
        self._brand = None
        self._model = None
        self._device = None
        self._cpu = None

        self.generate_device()

    def generate_device_id(self, seed):
        volatile_seed = "12345"
        m = hashlib.md5()
        m.update(seed.encode('utf-8') + volatile_seed.encode('utf-8'))
        return 'android-' + m.hexdigest()[:16]


    def get_random_device(self):
        return random.choice(self.DEVICES)

    def get_all_good_devices(self):
        return self.DEVICES

    def is_good_device(self, device: str):
        return True if device in self.DEVICES else False

    def generate_device(self):
        device = self.get_random_device()

        parts = device.split('; ')
        if len(parts) != 7:
            raise RuntimeError('Invalid Device: %s' % device)

        andriodOS = str(parts[0]).split('/', 2)
        if version.parse(andriodOS[1]) < version.parse(self.REQUIRED_ANDROID_VERSION):
            raise RuntimeError('Device %s required Andriod version %s for instagram' % (device, self.REQUIRED_ANDROID_VERSION))

        resolution = str(parts[2]).split('x', 2)
        pixelCount = int(resolution[0]) * int(resolution[1])
        if pixelCount < 2073600:
            raise RuntimeError('Invalid Resolution: %s minimum resolution is 1920x1080' % resolution)
        
        manufacturerAndBrand = str(parts[3]).split('/', 2)

        self._device = device
        self._androidVersion = andriodOS[0]
        self._androidRelease = andriodOS[1]
        self._dpi = parts[1]
        self._resolution = parts[2]
        self._manufacturer = manufacturerAndBrand[0]
        try:
            self._brand = manufacturerAndBrand[1] if manufacturerAndBrand[1] else None
        except IndexError:
            pass
        self._model = parts[4]
        self._device = parts[5]
        self._cpu = parts[6]

        self._userAgent = self.build_user_agent()


    def build_user_agent(self):
        manufacturer = self._manufacturer
        if self._brand != None:
            manufacturer = '/' + self._brand

        device = self.USER_AGENT_FORMAT % (
            _config.IG_VERSION,
            self._androidVersion,
            self._androidRelease,
            self._dpi,
            self._resolution,
            manufacturer,
            self._model,
            self._device,
            self._cpu,
            _config.USER_AGENT_LOCALE,
        )
        return device

        
    
    



from uuid import uuid4
import hmac
import urllib
import hashlib
from instapv.config import Config

class Tools:

    def __init__(self):
        self.config = Config()
        super().__init__()

    def generate_uuid(self, type):
        generated_uuid = str(uuid4())
        if (type):
            return generated_uuid
        else:
            return generated_uuid.replace('-', '')

    def generate_signature(self, data, skip_quote=False):
        if not skip_quote:
            try:
                parsedData = urllib.parse.quote(data)
            except AttributeError:
                parsedData = urllib.quote(data)
        else:
            parsedData = data
        return 'ig_sig_key_version=' + self.config.SIG_KEY_VERSION + '&signed_body=' + hmac.new(self.config.IG_SIG_KEY.encode('utf-8'), data.encode('utf-8'), hashlib.sha256).hexdigest() + '.' + parsedData




import os
import json
import uuid
import time
import pickle
import random
import hashlib
import requests
from instapv import DeviceGenerator, Config, Tools
from instapv.exceptions import *
from instapv.logger import Logger
from instapv.ig.user import User
from instapv.ig.media import Media
from instapv.ig.business import Business 
from instapv.ig.live import Live 
from instapv.ig.account import Account 
from urllib.parse import urlencode
from instapv.response.login import LoginResponse

# Ignore InsecureRequestWarning
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

# Logger for debugging
log = Logger()


if not os.path.exists('cache'):
    os.mkdir('cache')

class Bot:


    def __init__(self, username: str, password: str, debug: bool = False, login_cache = False):
        self.cache = hashlib.md5(username.encode('utf-8')).hexdigest()
        self.config = Config()
        self.tools = Tools()
        self.device = DeviceGenerator()
        self.is_logged_in = False
        self.last_response = None
        self.debug = debug
        self.login_cache = login_cache
        self.username = username
        if not os.path.exists(f'cache/{self.username}'):
            os.mkdir(f'cache/{self.username}')
        try:
            if os.path.exists(f'cache/{username}/{self.cache}.pkl'):
                if self.debug:
                    log.info('Loading session...')
                self.session(False)
            else:
                if self.debug:
                    log.warn('Creating new session ...')
                self.req = requests.Session()
                self.login_cache = False
        except IOError:
            if self.debug:
                log.warn('Creating new session ...')
            self.req = requests.Session()
        self.set_user(username, password)
        self.device_id = self.device.generate_device_id(self._s.hexdigest())

        # Call Classes
        self.user  = User(self)
        self.media = Media(self)
        self.business = Business(self)
        self.live = Live(self)
        self.account = Account(self)


    def set_proxy(self, proxy=None):
        if proxy is not None:
            proxies = {'http': proxy}
            self.req.proxies.update(proxies)
        else:
            self.req.proxies.clear()

    def set_user(self, username: str, password: str):
        self._s = hashlib.md5()
        self._s.update(username.encode('utf-8') + password.encode('utf-8'))
        self.username = username
        self.password = password
        self.uuid = self.tools.generate_uuid(True)

    def session(self, create=False):
        if create:
            with open(f'cache/{self.username}/{self.cache}.pkl', 'wb') as f:
                pickle.dump(self.req, f)
        else:
            with open(f'cache/{self.username}/{self.cache}.pkl', 'rb') as f:
                self.req = pickle.load(f)

    def fetch_headers(self):
        data = {
            'challenge_type': 'signup',
            'guid': self.tools.generate_uuid(False)
        }
        query = self.request('si/fetch_headers/', get_params=data, needs_auth=False)
        self.token = self.last_response.cookies['csrftoken']
        self.app_headers = {
            'phone_id': self.tools.generate_uuid(True),
            '_csrftoken': self.last_response.cookies['csrftoken'],
            'username': self.username,
            'guid': self.uuid,
            'device_id': self.device_id,
            'password': self.password,
            'login_attempt_count': '0'
        }
        return query

    def login(self, relogin=False):
        if self.login_cache and not relogin:
            with open(f'cache/{self.username}/data.json', 'r', encoding='utf-8') as _data:
                _dict = json.loads(_data.read())
            self.is_logged_in = _dict['is_logged_in']
            self.account_id = _dict['account_id']
            self.rank_token = _dict['rank_token']
            self.token = _dict['token']
            self.phone_id = _dict['phone_id']
            self.app_headers = _dict['app_headers']
            self.logged_in_user = _dict['logged_in_user']
            return LoginResponse(self.logged_in_user)
        else:
            if self.fetch_headers():
                query = self.request('accounts/login/', self.app_headers, needs_auth=False)
                _respone = LoginResponse(query)
                if _respone.status == 'ok':                    
                    self.phone_id = self.tools.generate_uuid(True)
                    self.is_logged_in = True
                    self.account_id = self.last_json_response["logged_in_user"]["pk"]
                    self.logged_in_user = self.last_json_response
                    self.rank_token = self.uuid
                    self.token = self.last_response.cookies["csrftoken"]
                    data = {
                        'phone_id': self.phone_id,
                        'is_logged_in': self.is_logged_in,
                        'account_id': self.account_id,
                        'rank_token': self.rank_token,
                        'token': self.token,
                        'logged_in_user': self.last_json_response,
                        'app_headers': self.app_headers
                    }
                    with open(f'cache/{self.username}/data.json', 'w', encoding='utf-8') as _f:
                        json.dump(data, _f, ensure_ascii=False, sort_keys=True ,indent=4)
                    if not os.path.exists(f'cache/{self.cache}.pkl'):
                        pass
                    if self.debug:
                        log.info(f'INFO: LOGGED IN AS {self.username}\n')
                    return _respone
                return _respone

    def send_two_factor_login_sms(self, username: str, password: str, two_factor_identifier: str, username_handler: str = None):
        username = username_handler if username_handler else username
        data = {
            'two_factor_identifier': two_factor_identifier,
            'username': username,
            'device_id': self.device_id,
            'guid': self.uuid,
            '_csrftoken': self.token
        }
        query = self.request('accounts/send_two_factor_login_sms/', params=data, needs_auth=False)
        return query

    def finish_two_factor_login(self, username: str, password: str, two_factor_identifier: str, verification_code: str, verification_method: str = '1', username_handler: str = None):
        # TODO: FIX THIS, CHECK verify_methods = ['1', '2', '3']
    
        verification_code = "".join(verification_code.split())
        username = username_handler if username_handler else username

        data = {
            'verification_method': verification_method,
            'verification_code': verification_code,
            'trust_this_device': 1,
            'two_factor_identifier': two_factor_identifier,
            '_csrftoken': self.token,
            'username': username,
            'device_id': self.device_id,
            'guid': self.uuid
        }
        query = self.request('accounts/two_factor_login/', params=data, needs_auth=False)
        _response = LoginResponse(query)
        if _response.status == "ok":
            self.fetch_headers()
            return _response
        return _response

    def sync(self):
        data = {
            '_uuid': self.uuid,
            '_uid': self.account_id,
            'id': self.account_id,
            '_csrftoken': self.token,
            'experiments': self.config.EXPERIMENTS
        }
        return self.request('qe/sync/', params=data)

    def load_user_list(self):
        return self.request('friendships/autocomplete_user_list/')

    def time_line_feed(self):
        return self.request('feed/timeline/')
    
    def get_inbox(self):
        inbox = self.request('direct_v2/inbox/?')
        return inbox

    def get_activity(self):
        activity = self.request('news/inbox/?')
        return activity

    def request(self, endpoint: str, params: dict = None, needs_auth: bool = True, signed_post: bool = True, files = None, get_params: dict = None):
        if self.debug:
            log.info(f"REQUEST: {self.config.API_URL + endpoint}")

        if not self.is_logged_in and needs_auth:
            return False

        self.req.headers.update({'Connection': 'close',
            'Accept': '*/*',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Cookie2': '$Version=1',
            'Accept-Language': 'en-US',
            'User-Agent': self.device.build_user_agent()
        })

        if params:
            if signed_post:
                params = self.tools.generate_signature(
                    json.dumps(params)
                )
            if files:
                self.req.headers.update({
                    'Content-Type': 'application/octet-stream',
                    'Content-Transfer-Encoding': 'binary'
                })
                response = self.req.post(self.config.API_URL + endpoint, files=files, data=params, verify=True)
            else:
                response = self.req.post(self.config.API_URL + endpoint, data=params, verify=True)
        else:
            if get_params:
                response = self.req.get(self.config.API_URL + endpoint, params=urlencode(get_params), verify=True)
            else:
                response = self.req.get(self.config.API_URL + endpoint, params=get_params , verify=True)
        if self.debug:
            log.info(f'CODE: {str(response.status_code)}', True)
            log.info(f"RESPONSE: {response.text}\n")

        if response.status_code == 200:
            self.last_response = response
            self.last_json_response = json.loads(response.text)
            self.session(True)
            return self.last_json_response
        elif response.status_code == 405:
            raise AccessDeniedException(f'Access deniend to request: {self.config.API_URL + endpoint}')
        else:
            if (self.debug):
                log.error(f'CODE: {str(response.status_code)}')
            try:
                self.last_response = response
                self.last_json_response = json.loads(response.text)
                if self.last_json_response['message'] == 'login_required':
                    if os.path.exists(f'cache/{self.username}/{self.cache}.pkl'):
                        os.remove(f'cache/{self.username}/{self.cache}.pkl')
                        os.rmdir(f'cache/{self.username}')
                    raise LogedOutException('Ssaved account password has changed and the old session has been deleted. Please log back in.')
                if self.last_json_response['message'] == 'challenge_required':
                    raise ChallengeRequiredException('Challenge required')
                if self.last_json_response['message'] == 'The password you entered is incorrect. Please try again.':
                    os.remove(f'cache/{self.username}/{self.cache}.pkl')
                    raise InvalidCredentialsException(self.last_json_response['message'])
                if (self.debug):
                    log.info(f'RESPONSE: {str(self.last_json_response)}')
            except SentryBlockException:
                raise
            return self.last_json_response



import json

class GenericResponse:

    def __init__(self, data: dict):
        if data:
            self.message = None
            self.status = None
            self.as_json = data
            
            if 'message' in data:
                self.message = data['message']
            if 'status' in data:
                self.status = data['status']



import json

class UserCommentResponse:

    def __init__(self, data: dict):
        self.pk = data['pk']
        self.username = data['username']
        self.full_name = data['full_name']
        self.is_private = data['is_private']
        self.profile_pic_url = data['profile_pic_url']
        self.is_verified = data['is_verified']
        self.has_anonymous_profile_picture = data['has_anonymous_profile_picture']
        self.reel_auto_archive = data['reel_auto_archive']
        self.allowed_commenter_type = data['allowed_commenter_type']

class SendCommentInfoResponse:

    def __init__(self, data: dict):
        self.as_json = data
        self.status = data['status']
        self.content_type = None
        self.user = None
        self.pk = None
        self.text = None
        self.type = None
        self.created_at = None
        self.created_at_utc = None
        self.media_id = None
        self.comment_status = None
        self.share_enabled = None

        if 'content_type' in data['comment']:
            self.content_type = data['comment']['content_type']
        if 'user' in data['comment']:
            self.user = UserCommentResponse(data['comment']['user'])
        if 'pk' in data['comment']:
            self.pk = data['comment']['pk']
        if 'text' in data['comment']:
            self.text = data['comment']['text']
        if 'type' in data['comment']:
            self.type = data['comment']['type']
        if 'created_at' in data['comment']:
            self.created_at = data['comment']['created_at']
        if 'created_at_utc' in data['comment']:
            self.created_at_utc = data['comment']['created_at_utc']
        if 'comment_status' in data['comment']:
            self.comment_status = data['comment']['status']
        if 'share_enabled' in data['comment']:
            self.share_enabled = data['comment']['share_enabled']



import json
from ..response.media_items_info import MediaItemsInfoResponse

class MediaInfoResponse:


    def __init__(self, data: dict):
        if 'num_results' in data:
            if int(data['num_results']) > 0:
                if 'items' in data:
                    self.items: MediaItemsInfoResponse = MediaItemsInfoResponse(data['items'][0])
                    self.num_results: int = data['num_results']
                    self.more_available: bool = data['more_available']
                    self.auto_load_more_enabled: bool = data['auto_load_more_enabled']
                    self.status: str = data['status']



import struct
import imghdr


def getImageSize(fname):
    with open(fname, 'rb') as fhandle:
        head = fhandle.read(24)
        if len(head) != 24:
            raise RuntimeError("Invalid Header")
        if imghdr.what(fname) == 'png':
            check = struct.unpack('>i', head[4:8])[0]
            if check != 0x0d0a1a0a:
                raise RuntimeError("PNG: Invalid check")
            width, height = struct.unpack('>ii', head[16:24])
        elif imghdr.what(fname) == 'gif':
            width, height = struct.unpack('<HH', head[6:10])
        elif imghdr.what(fname) == 'jpeg':
            fhandle.seek(0)  # Read 0xff next
            size = 2
            ftype = 0
            while not 0xc0 <= ftype <= 0xcf:
                fhandle.seek(size, 1)
                byte = fhandle.read(1)
                while ord(byte) == 0xff:
                    byte = fhandle.read(1)
                ftype = ord(byte)
                size = struct.unpack('>H', fhandle.read(2))[0] - 2
            # We are at a SOFn block
            fhandle.seek(1, 1)  # Skip `precision' byte.
            height, width = struct.unpack('>HH', fhandle.read(4))
        else:
            raise RuntimeError("Unsupported format")
        return width, height



import json
from instapv.exceptions import UserNotFoundException


class SelfUserFeedResponse:
    
    def __init__(self, data):
        if data:
            self.pk = None
            self.username = None
            self.full_name = None
            self.profile_pic_url = None
            self.is_verified = None
            self.is_private = None
            self.has_anonymous_profile_picture = None
            self.can_boost_post = None
            self.can_see_organic_insights = None
            self.show_insights_terms = None
            self.reel_auto_archive = None
            self.is_unpublished = None
            self.latest_reel_media = None

            if type(data) == dict:
                if 'items' in data:
                    self.data = data['items'][0]['user']
                else:
                    self.data = data['user']
            # Check request status 
            if 'pk' in self.data:
                self.pk = self.data['pk']
            if 'username' in self.data:
                self.username = self.data['username']
            if 'full_name' in self.data:
                self.full_name = self.data['full_name']
            if 'is_private' in self.data:
                self.is_private = self.data['is_private']
            if 'profile_pic_url' in self.data:
                self.profile_pic_url = self.data['profile_pic_url']
            if 'is_verified' in self.data:
                self.is_verified = self.data['is_verified']
            if 'has_anonymous_profile_picture' in self.data:
                self.has_anonymous_profile_picture = self.data['has_anonymous_profile_picture']
            if 'can_boost_post' in self.data:
                self.can_boost_post = self.data['can_boost_post']
            if 'can_see_organic_insights' in self.data:
                self.can_see_organic_insights = self.data['can_see_organic_insights']
            if 'show_insights_terms' in self.data:
                self.show_insights_terms = self.data['show_insights_terms']
            if 'reel_auto_archive' in self.data:
                self.reel_auto_archive = self.data['reel_auto_archive']
            if 'is_unpublished' in self.data:
                self.is_unpublished = self.data['is_unpublished']
            if 'allowed_commenter_type' in self.data:
                self.allowed_commenter_type = self.data['allowed_commenter_type']
            if 'latest_reel_media' in self.data:
                self.latest_reel_media = self.data['latest_reel_media']


import json

from instapv.response.two_factor_info import TwoFactorResponse
from instapv.response.user_response import UserResponse

class PhoneVerificationSettingsResponse:

    def __init__(self, data: dict):
        # Return all data as_json
        self.as_json = data
        
        self.max_sms_count = data['max_sms_count'] if 'max_sms_count' in data else None
        self.resend_sms_delay_sec = data['resend_sms_delay_sec'] if 'resend_sms_delay_sec' in data else None
        self.robocall_count_down_time_sec = data['robocall_count_down_time_sec'] if 'robocall_count_down_time_sec' in data else False
        self.robocall_after_max_sms = data['robocall_after_max_sms'] if 'robocall_after_max_sms' in data else None

class LoginResponse:

    def __init__(self, data: dict):
        # Return all data as_json
        self.as_json = data
        
        self.message = data['message'] if 'message' in data else None
        self.status = data['status'] if 'status' in data else None
        self.two_factor_required = data['two_factor_required'] if 'two_factor_required' in data else False
        self.two_factor_info = TwoFactorResponse(data['two_factor_info']) if 'two_factor_info' in data else None
        self.phone_verification_settings = PhoneVerificationSettingsResponse(data['phone_verification_settings']) if 'phone_verification_settings' in data else None
        self.logged_in_user = UserResponse(data['logged_in_user']) if 'logged_in_user' in data else False


import json
from instapv.exceptions import UserNotFoundException


class FriendShipResponse:
    
    def __init__(self, data):
        if data:
            self.status = None
            self.following = None
            self.followed_by = None
            self.blocking = None
            self.muting = None
            self.is_private = None
            self.incoming_request = None
            self.outgoing_request = None
            self.is_bestie = None
            self.is_restricted = None

            # Check request status 
            if 'status' in data:
                if data['status'] != 'ok':
                    raise UserNotFoundException('UserNotFound')
                if 'friendship_status' in data:
                    data = data['friendship_status']
                if 'following' in data:
                    self.following = data['following']
                if 'followed_by' in data:
                    self.followed_by = data['followed_by']
                if 'blocking' in data:
                    self.blocking = data['blocking']
                if 'muting' in data:
                    self.muting = data['muting']
                if 'is_private' in data:
                    self.is_private = data['is_private']
                if 'incoming_request' in data:
                    self.incoming_request = data['incoming_request']
                if 'outgoing_request' in data:
                    self.outgoing_request = data['outgoing_request']
                if 'is_bestie' in data:
                    self.is_bestie = data['is_bestie']
                if 'is_restricted' in data:
                    self.is_restricted = data['is_restricted']
                if 'status' in data:
                    self.status = data['status']
            else:
                raise UserNotFoundException('UserNotFound')



import json
from datetime import datetime

from instapv.response.self_user_feed import SelfUserFeedResponse
from instapv.response.generic import GenericResponse
from instapv.exceptions import InternalException

class Account:

    def __init__(self, bot):
        self.bot = bot

    def set_private(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request('accounts/set_private/', data)
        return GenericResponse(query)

    def set_public(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request('accounts/set_public/', data)
        return GenericResponse(query)

    def get_current_user(self):
        query = self.bot.request('accounts/current_user/?edit=true')
        return SelfUserFeedResponse(query)

    def set_biography(self, biography: str):
        if not isinstance(biography, str) or len(biography) > 150:
            raise Exception('Please provide a 0 to 150 character string as biography.')
        else:
            data = {
                'raw_text': biography,
                '_uuid': self.bot.uuid,
                '_uid': self.bot.account_id,
                'device_id': self.bot.device_id,
                '_csrftoken': self.bot.token
            }
            query = self.bot.request('accounts/set_biography/', params=data)
            return query
    
    def set_gender(self, gender: str = ''):
        switcher = { 
            "male": 1, 
            "female": 2, 
            " ": 3 
        }
        gender_id = switcher.get(gender, 4)
        custom_gender = gender if gender_id == 4 else '' 
        data = {
            'gender': gender_id,
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid,
            'custom_gender': custom_gender
        }
        query = self.bot.request('accounts/set_gender/', params=data, signed_post=False)
        return query

    def edit_profile(self, url: str, phone_number: str, name: str, biography: str, email: str, gender: str, new_username = None):
        user_respone = self.get_current_user()
        current_user = user_respone
        if not current_user or not isinstance(current_user.username, str):
            raise InternalException('Unable to find current account username while preparing profile edit.')
        
        old_username = current_user.username
        if isinstance(new_username, str) and len(new_username) > 0:
            username = new_username
        else:
            username = old_username

        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'external_url': url,
            'phone_number': phone_number,
            'username': username,
            'first_name': name,
            'biography': biography,
            'email': email,
            'gender': gender,
            'device_id': self.bot.device_id
        }
        query = self.bot.request('accounts/edit_profile/', params=data)
        return query


    def change_profile_picture(self, photo_filename):
        """
        TODO: FIX FILE UPLOAD
        """
        data = {
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
        }
        _photo = {
            'profile_pic': (photo_filename, open(photo_filename, 'rb'), 'application/octet-stream')
        }
        query = self.bot.request('accounts/change_profile_picture/', params=data, files=_photo, signed_post=False)
        return query
    
    def remove_profile_picture(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/remove_profile_picture/', params=data)
        return query

    def switch_to_business_profile(self):
        """
        Switches your account to business profile.
        In order to switch your account to Business profile you MUST call self.set_business_info
        """
        return self.bot.request('business_conversion/get_business_convert_social_context/')

    def switch_to_personal_profile(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/convert_to_personal/', params=data)
        return query
    
    def set_business_info(self, phone_number, email, category_id):
        data = {
            'set_public': 'true',
            'entry_point': 'setting',
            'public_phone_contact': json.dumps({
                'public_phone_number': phone_number,
                'business_contact_method': 'CALL'
            }),
            'public_email': email,
            'category_id': category_id,
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request('accounts/create_business_info/', params=data)
        return query

    def check_username(self, username):
        data = {
            '_uuid': self.bot.uuid,
            'username': username,
            '_csrftoken': self.bot.token,
            '_uid': self.bot.account_id,
        }
        query = self.bot.request('users/check_username/', params=data)
        return query

    def get_comment_filter(self):
        return self.bot.request('accounts/get_comment_filter/')
    
    def set_comment_filter(self, config_value: int):
        if not isinstance(config_value, int):
            raise ValueError('config_value must be 1 for on or 0 for off.')
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'config_value': config_value,
        }
        query = self.bot.request('accounts/set_comment_filter/', params=data)
        return query
    
    def get_comment_category_filter_disabled(self):
        return self.bot.request('accounts/get_comment_category_filter_disabled/')
    
    def get_comment_filter_keywords(self):
        return self.bot.request('accounts/get_comment_filter_keywords/')
    
    def set_comment_filter_keywords(self, keywords: list):
        if not isinstance(keywords, list):
            raise ValueError('Keywords must be instance of list')
        keywords = ','.join(keywords)
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'keywords': keywords,
        }
        query = self.bot.request('accounts/set_comment_filter_keywords/', params=data)
        return query

    def change_password(self, old_password, new_password):
        """
        WARRNING: You should call bot.login(relogin=True) to relogin with new password!
        """
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'old_password': old_password,
            'new_password1': new_password,
            'new_password2': new_password,
        }
        query = self.bot.request('accounts/change_password/', params=data)
        return query
    
    def get_security_info(self):
        """
        WARNING: STORE AND KEEP BACKUP CODES IN A SAFE PLACE. THEY ARE EXTREMELY
        IMPORTANT! YOU WILL GET THE CODES IN THE RESPONSE. THE BACKUP
        CODES LET YOU REGAIN CONTROL OF YOUR ACCOUNT IF YOU LOSE THE
        PHONE NUMBER! WITHOUT THE CODES, YOU RISK LOSING YOUR ACCOUNT!
        """
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/account_security_info/', params=data)
        return query
    
    def send_two_factor_enable_sms(self, phone_number: str):
        clean_number = phone_number.replace(r"[^0-9]", "")
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'device_id': self.bot.device_id,
            'phone_number': clean_number
        }
        query = self.bot.request('accounts/send_two_factor_enable_sms/', params=data)
        return query

    def enable_sms_two_factor(self, phone_number: str, verification_code):
        clean_number = phone_number.replace(r"[^0-9]", "")
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'device_id': self.bot.device_id,
            'phone_number': clean_number,
            'verification_code': verification_code
        }
        query = self.bot.request('accounts/enable_sms_two_factor/', params=data)
        return query
    
    def disable_sms_two_factor(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/disable_sms_two_factor/', params=data)
        return query
    
    def get_presence_status(self):
        return self.bot.request('accounts/get_presence_disabled/')
    
    def enable_presence(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'disabled': '0',
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/set_presence_disabled/', params=data)
        return query
    
    def disable_presence(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'disabled': '1',
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/set_presence_disabled/', params=data)
        return query
    
    def send_confirm_email(self):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'send_source': 'edit_profile',
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/send_confirm_email/', params=data)
        return query
    
    def send_sms_code(self, phone_number: str):
        clean_number = phone_number.replace(r"[^0-9]", "")
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'phone_number': clean_number,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request('accounts/send_sms_code/', params=data)
        return query
    
    def verify_sms_code(self, phone_number: str, verification_code):
        clean_number = phone_number.replace(r"[^0-9]", "")
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'phone_number': clean_number,
            'verification_code': verification_code
        }
        query = self.bot.request('accounts/verify_sms_code/', params=data)
        return query
    
    def set_contact_point_prefill(self, usage):
        data = {
            'phone_id': self.bot.phone_id,
            '_csrftoken': self.bot.token,
            'usage': usage,
        }
        query = self.bot.request('accounts/contact_point_prefill/', params=data)
        return query
    
    def get_badge_notifications(self):
        data = {
            '_uuid': self.bot.phone_id,
            '_csrftoken': self.bot.token,
            'user_ids': self.bot.account_id,
            'phone_id': self.bot.phone_id,
        }
        query = self.bot.request('notifications/badge/', params=data, signed_post=False)
        return query
    
    def get_process_contact_point_signals(self):
        data = {
            'google_tokens': '[]',
            'phone_id': self.bot.token,
            '_uid': self.bot.account_id,
            '_uuid': self.bot.uuid,
            'device_id': self.bot.device_id,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request('accounts/process_contact_point_signals/', params=data)
        return query
    
    def get_prefill_candidates(self):
        data = {
            'android_device_id': self.bot.device_id,
            'device_id': self.bot.uuid,
            'usages': '["account_recovery_omnibox"]',
        }
        query = self.bot.request('accounts/get_prefill_candidates/', params=data)
        return query
    
    def get_account_family(self):
        return self.bot.request('multiple_accounts/get_account_family/')
    
    def get_linkage_status(self):
        return self.bot.request('linked_accounts/get_linkage_status/')



"""
Config File


"""

class Config:

    API_URL = 'https://i.instagram.com/api/v1/'
    IG_SIG_KEY = '4f8732eb9ba7d1c8e8897a75d6474d4eb3f5279137431b2aafb71fafe2abe178'
    IG_VERSION = '107.0.0.27.121'
    VERSION_CODE = '168361634'

    USER_AGENT_LOCALE = 'en_US'

    EXPERIMENTS = 'ig_promote_reach_objective_fix_universe,ig_android_universe_video_production,ig_search_client_h1_2017_holdout,ig_android_live_follow_from_comments_universe,ig_android_carousel_non_square_creation,ig_android_live_analytics,ig_android_follow_all_dialog_confirmation_copy,ig_android_stories_server_coverframe,ig_android_video_captions_universe,ig_android_offline_location_feed,ig_android_direct_inbox_retry_seen_state,ig_android_ontact_invite_universe,ig_android_live_broadcast_blacklist,ig_android_insta_video_reconnect_viewers,ig_android_ad_async_ads_universe,ig_android_search_clear_layout_universe,ig_android_shopping_reporting,ig_android_stories_surface_universe,ig_android_verified_comments_universe,ig_android_preload_media_ahead_in_current_reel,android_instagram_prefetch_suggestions_universe,ig_android_reel_viewer_fetch_missing_reels_universe,ig_android_direct_search_share_sheet_universe,ig_android_business_promote_tooltip,ig_android_direct_blue_tab,ig_android_async_network_tweak_universe,ig_android_elevate_main_thread_priority_universe,ig_android_stories_gallery_nux,ig_android_instavideo_remove_nux_comments,ig_video_copyright_whitelist,ig_react_native_inline_insights_with_relay,ig_android_direct_thread_message_animation,ig_android_draw_rainbow_client_universe,ig_android_direct_link_style,ig_android_live_heart_enhancements_universe,ig_android_rtc_reshare,ig_android_preload_item_count_in_reel_viewer_buffer,ig_android_users_bootstrap_service,ig_android_auto_retry_post_mode,ig_android_shopping,ig_android_main_feed_seen_state_dont_send_info_on_tail_load,ig_fbns_preload_default,ig_android_gesture_dismiss_reel_viewer,ig_android_tool_tip,ig_android_ad_logger_funnel_logging_universe,ig_android_gallery_grid_column_count_universe,ig_android_business_new_ads_payment_universe,ig_android_direct_links,ig_android_audience_control,ig_android_live_encore_consumption_settings_universe,ig_perf_android_holdout,ig_android_cache_contact_import_list,ig_android_links_receivers,ig_android_ad_impression_backtest,ig_android_list_redesign,ig_android_stories_separate_overlay_creation,ig_android_stop_video_recording_fix_universe,ig_android_render_video_segmentation,ig_android_live_encore_reel_chaining_universe,ig_android_sync_on_background_enhanced_10_25,ig_android_immersive_viewer,ig_android_mqtt_skywalker,ig_fbns_push,ig_android_ad_watchmore_overlay_universe,ig_android_react_native_universe,ig_android_profile_tabs_redesign_universe,ig_android_live_consumption_abr,ig_android_story_viewer_social_context,ig_android_hide_post_in_feed,ig_android_video_loopcount_int,ig_android_enable_main_feed_reel_tray_preloading,ig_android_camera_upsell_dialog,ig_android_ad_watchbrowse_universe,ig_android_internal_research_settings,ig_android_search_people_tag_universe,ig_android_react_native_ota,ig_android_enable_concurrent_request,ig_android_react_native_stories_grid_view,ig_android_business_stories_inline_insights,ig_android_log_mediacodec_info,ig_android_direct_expiring_media_loading_errors,ig_video_use_sve_universe,ig_android_cold_start_feed_request,ig_android_enable_zero_rating,ig_android_reverse_audio,ig_android_branded_content_three_line_ui_universe,ig_android_live_encore_production_universe,ig_stories_music_sticker,ig_android_stories_teach_gallery_location,ig_android_http_stack_experiment_2017,ig_android_stories_device_tilt,ig_android_pending_request_search_bar,ig_android_fb_topsearch_sgp_fork_request,ig_android_seen_state_with_view_info,ig_android_animation_perf_reporter_timeout,ig_android_new_block_flow,ig_android_story_tray_title_play_all_v2,ig_android_direct_address_links,ig_android_stories_archive_universe,ig_android_save_collections_cover_photo,ig_android_live_webrtc_livewith_production,ig_android_sign_video_url,ig_android_stories_video_prefetch_kb,ig_android_stories_create_flow_favorites_tooltip,ig_android_live_stop_broadcast_on_404,ig_android_live_viewer_invite_universe,ig_android_promotion_feedback_channel,ig_android_render_iframe_interval,ig_android_accessibility_logging_universe,ig_android_camera_shortcut_universe,ig_android_use_one_cookie_store_per_user_override,ig_profile_holdout_2017_universe,ig_android_stories_server_brushes,ig_android_ad_media_url_logging_universe,ig_android_shopping_tag_nux_text_universe,ig_android_comments_single_reply_universe,ig_android_stories_video_loading_spinner_improvements,ig_android_collections_cache,ig_android_comment_api_spam_universe,ig_android_facebook_twitter_profile_photos,ig_android_shopping_tag_creation_universe,ig_story_camera_reverse_video_experiment,ig_android_direct_bump_selected_recipients,ig_android_ad_cta_haptic_feedback_universe,ig_android_vertical_share_sheet_experiment,ig_android_family_bridge_share,ig_android_search,ig_android_insta_video_consumption_titles,ig_android_stories_gallery_preview_button,ig_android_fb_auth_education,ig_android_camera_universe,ig_android_me_only_universe,ig_android_instavideo_audio_only_mode,ig_android_user_profile_chaining_icon,ig_android_live_video_reactions_consumption_universe,ig_android_stories_hashtag_text,ig_android_post_live_badge_universe,ig_android_swipe_fragment_container,ig_android_search_users_universe,ig_android_live_save_to_camera_roll_universe,ig_creation_growth_holdout,ig_android_sticker_region_tracking,ig_android_unified_inbox,ig_android_live_new_watch_time,ig_android_offline_main_feed_10_11,ig_import_biz_contact_to_page,ig_android_live_encore_consumption_universe,ig_android_experimental_filters,ig_android_search_client_matching_2,ig_android_react_native_inline_insights_v2,ig_android_business_conversion_value_prop_v2,ig_android_redirect_to_low_latency_universe,ig_android_ad_show_new_awr_universe,ig_family_bridges_holdout_universe,ig_android_background_explore_fetch,ig_android_following_follower_social_context,ig_android_video_keep_screen_on,ig_android_ad_leadgen_relay_modern,ig_android_profile_photo_as_media,ig_android_insta_video_consumption_infra,ig_android_ad_watchlead_universe,ig_android_direct_prefetch_direct_story_json,ig_android_shopping_react_native,ig_android_top_live_profile_pics_universe,ig_android_direct_phone_number_links,ig_android_stories_weblink_creation,ig_android_direct_search_new_thread_universe,ig_android_histogram_reporter,ig_android_direct_on_profile_universe,ig_android_network_cancellation,ig_android_background_reel_fetch,ig_android_react_native_insights,ig_android_insta_video_audio_encoder,ig_android_family_bridge_bookmarks,ig_android_data_usage_network_layer,ig_android_universal_instagram_deep_links,ig_android_dash_for_vod_universe,ig_android_modular_tab_discover_people_redesign,ig_android_mas_sticker_upsell_dialog_universe,ig_android_ad_add_per_event_counter_to_logging_event,ig_android_sticky_header_top_chrome_optimization,ig_android_rtl,ig_android_biz_conversion_page_pre_select,ig_android_promote_from_profile_button,ig_android_live_broadcaster_invite_universe,ig_android_share_spinner,ig_android_text_action,ig_android_own_reel_title_universe,ig_promotions_unit_in_insights_landing_page,ig_android_business_settings_header_univ,ig_android_save_longpress_tooltip,ig_android_constrain_image_size_universe,ig_android_business_new_graphql_endpoint_universe,ig_ranking_following,ig_android_stories_profile_camera_entry_point,ig_android_universe_reel_video_production,ig_android_power_metrics,ig_android_sfplt,ig_android_offline_hashtag_feed,ig_android_live_skin_smooth,ig_android_direct_inbox_search,ig_android_stories_posting_offline_ui,ig_android_sidecar_video_upload_universe,ig_android_promotion_manager_entry_point_universe,ig_android_direct_reply_audience_upgrade,ig_android_swipe_navigation_x_angle_universe,ig_android_offline_mode_holdout,ig_android_live_send_user_location,ig_android_direct_fetch_before_push_notif,ig_android_non_square_first,ig_android_insta_video_drawing,ig_android_swipeablefilters_universe,ig_android_live_notification_control_universe,ig_android_analytics_logger_running_background_universe,ig_android_save_all,ig_android_reel_viewer_data_buffer_size,ig_direct_quality_holdout_universe,ig_android_family_bridge_discover,ig_android_react_native_restart_after_error_universe,ig_android_startup_manager,ig_story_tray_peek_content_universe,ig_android_profile,ig_android_high_res_upload_2,ig_android_http_service_same_thread,ig_android_scroll_to_dismiss_keyboard,ig_android_remove_followers_universe,ig_android_skip_video_render,ig_android_story_timestamps,ig_android_live_viewer_comment_prompt_universe,ig_profile_holdout_universe,ig_android_react_native_insights_grid_view,ig_stories_selfie_sticker,ig_android_stories_reply_composer_redesign,ig_android_streamline_page_creation,ig_explore_netego,ig_android_ig4b_connect_fb_button_universe,ig_android_feed_util_rect_optimization,ig_android_rendering_controls,ig_android_os_version_blocking,ig_android_encoder_width_safe_multiple_16,ig_search_new_bootstrap_holdout_universe,ig_android_snippets_profile_nux,ig_android_e2e_optimization_universe,ig_android_comments_logging_universe,ig_shopping_insights,ig_android_save_collections,ig_android_live_see_fewer_videos_like_this_universe,ig_android_show_new_contact_import_dialog,ig_android_live_view_profile_from_comments_universe,ig_fbns_blocked,ig_formats_and_feedbacks_holdout_universe,ig_android_reduce_view_pager_buffer,ig_android_instavideo_periodic_notif,ig_search_user_auto_complete_cache_sync_ttl,ig_android_marauder_update_frequency,ig_android_suggest_password_reset_on_oneclick_login,ig_android_promotion_entry_from_ads_manager_universe,ig_android_live_special_codec_size_list,ig_android_enable_share_to_messenger,ig_android_background_main_feed_fetch,ig_android_live_video_reactions_creation_universe,ig_android_channels_home,ig_android_sidecar_gallery_universe,ig_android_upload_reliability_universe,ig_migrate_mediav2_universe,ig_android_insta_video_broadcaster_infra_perf,ig_android_business_conversion_social_context,android_ig_fbns_kill_switch,ig_android_live_webrtc_livewith_consumption,ig_android_destroy_swipe_fragment,ig_android_react_native_universe_kill_switch,ig_android_stories_book_universe,ig_android_all_videoplayback_persisting_sound,ig_android_draw_eraser_universe,ig_direct_search_new_bootstrap_holdout_universe,ig_android_cache_layer_bytes_threshold,ig_android_search_hash_tag_and_username_universe,ig_android_business_promotion,ig_android_direct_search_recipients_controller_universe,ig_android_ad_show_full_name_universe,ig_android_anrwatchdog,ig_android_qp_kill_switch,ig_android_2fac,ig_direct_bypass_group_size_limit_universe,ig_android_promote_simplified_flow,ig_android_share_to_whatsapp,ig_android_hide_bottom_nav_bar_on_discover_people,ig_fbns_dump_ids,ig_android_hands_free_before_reverse,ig_android_skywalker_live_event_start_end,ig_android_live_join_comment_ui_change,ig_android_direct_search_story_recipients_universe,ig_android_direct_full_size_gallery_upload,ig_android_ad_browser_gesture_control,ig_channel_server_experiments,ig_android_video_cover_frame_from_original_as_fallback,ig_android_ad_watchinstall_universe,ig_android_ad_viewability_logging_universe,ig_android_new_optic,ig_android_direct_visual_replies,ig_android_stories_search_reel_mentions_universe,ig_android_threaded_comments_universe,ig_android_mark_reel_seen_on_Swipe_forward,ig_internal_ui_for_lazy_loaded_modules_experiment,ig_fbns_shared,ig_android_capture_slowmo_mode,ig_android_live_viewers_list_search_bar,ig_android_video_single_surface,ig_android_offline_reel_feed,ig_android_video_download_logging,ig_android_last_edits,ig_android_exoplayer_4142,ig_android_post_live_viewer_count_privacy_universe,ig_android_activity_feed_click_state,ig_android_snippets_haptic_feedback,ig_android_gl_drawing_marks_after_undo_backing,ig_android_mark_seen_state_on_viewed_impression,ig_android_live_backgrounded_reminder_universe,ig_android_live_hide_viewer_nux_universe,ig_android_live_monotonic_pts,ig_android_search_top_search_surface_universe,ig_android_user_detail_endpoint,ig_android_location_media_count_exp_ig,ig_android_comment_tweaks_universe,ig_android_ad_watchmore_entry_point_universe,ig_android_top_live_notification_universe,ig_android_add_to_last_post,ig_save_insights,ig_android_live_enhanced_end_screen_universe,ig_android_ad_add_counter_to_logging_event,ig_android_blue_token_conversion_universe,ig_android_exoplayer_settings,ig_android_progressive_jpeg,ig_android_offline_story_stickers,ig_android_gqls_typing_indicator,ig_android_chaining_button_tooltip,ig_android_video_prefetch_for_connectivity_type,ig_android_use_exo_cache_for_progressive,ig_android_samsung_app_badging,ig_android_ad_holdout_watchandmore_universe,ig_android_offline_commenting,ig_direct_stories_recipient_picker_button,ig_insights_feedback_channel_universe,ig_android_insta_video_abr_resize,ig_android_insta_video_sound_always_on'''
    SIG_KEY_VERSION = '4'

    ALPHABET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_'



import json
from instapv.response.user_response import UserResponse
from instapv.response.friendship import FriendShipResponse
from instapv.response.self_user_feed import SelfUserFeedResponse

class User:

    def __init__(self, bot):
        self.bot = bot

    def get_user_feed(self, user_id: str, max_id: str = None, timestamp: str = None):
        data = {
            'max_id': max_id,
            'min_timestamp': timestamp,
            'rank_token': self.bot.tools.generate_uuid(True),
            'ranked_token': 'true'
        }

        query = self.bot.request(f'feed/user/{user_id}/', params=data)
        return UserResponse(query)

    def get_self_user_feed(self, max_id: str = None, timestamp: str = None):
        data = {
            'max_id': max_id,
            'min_timestamp': timestamp,
            'rank_token': self.bot.tools.generate_uuid(True),
            'ranked_token': 'true'
        }

        query = self.bot.request(f'feed/user/{self.bot.account_id}/', params=data)
        return SelfUserFeedResponse(query)

    def get_info_by_name(self, username):
        query = self.bot.request(f'users/{username}/usernameinfo/')
        return UserResponse(query)

    def follow_request_approve(self, user_id):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'user_id': user_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request(f'friendships/approve/{user_id}/', params=data, signed_post=True)
        return FriendShipResponse(query)

    def follow_request_ignore(self, user_id):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'user_id': user_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request(f'friendships/ignore/{user_id}/', params=data, signed_post=True)
        return FriendShipResponse(query)

    def follow(self, user_id, media_id=None):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'user_id': user_id,
            'radio_type': 'wifi-none',
            'device_id': self.bot.device_id
        }
        if media_id:
            data.update({'media_id_attribution': media_id})
        query = self.bot.request(f'friendships/create/{user_id}/', params=data, signed_post=True)
        return FriendShipResponse(query)

    def unfollow(self, user_id):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'user_id': user_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request(f'friendships/destroy/{user_id}/', params=data, signed_post=True)
        return FriendShipResponse(query)

    def block(self, user_id):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'user_id': user_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request(f'friendships/block/{user_id}/', params=data, signed_post=True)
        return FriendShipResponse(query)

    def unblock(self, user_id):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'user_id': user_id,
            '_csrftoken': self.bot.token
        }
        query = self.bot.request(f'friendships/unblock/{user_id}/', params=data, signed_post=True)
        return FriendShipResponse(query)

    def get_user_followers(self, user_id, max_id: str = None):
        if max_id == None:
            query = self.bot.request(f'friendships/{user_id}/followers/?rank_token=' + self.bot.tools.generate_uuid(True))
            return query
        query = self.bot.request(f'friendships/{user_id}/followers/?rank_token={self.bot.tools.generate_uuid(True)}&max_id={max_id}')
        return query

    def get_pending_follow_requests(self):
        query = self.bot.request('friendships/pending?')
        return UserResponse(query)

    def get_user_following(self, user_id, max_id: str = None):
        if max_id == None:
            query = self.bot.request(f'friendships/{user_id}/following/?rank_token=' + self.bot.tools.generate_uuid(True))
            return UserResponse(query['users'])
        query = self.bot.request(f'friendships/{user_id}/following/?rank_token={self.bot.tools.generate_uuid(True)}&max_id={max_id}')
        return UserResponse(query['users'])

    def get_all_followers(self, user_id):
        followers = []
        next_max_id: str = None
        while 1:
            self.get_user_followers(user_id, next_max_id)
            temp = self.bot.last_json_response

            for item in temp["users"]:
                followers.append(item)

            if temp["big_list"] is False:
                return followers
            next_max_id = temp["next_max_id"]



from colorama import Fore, Back
from datetime import datetime
from time import sleep

class Logger:
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'


    def warn(self, text: str, ask: bool = False, error_code: int = None):
        now = datetime.now()
        CLOCK = '{}[{}{}{}{}{}]{}{} '.format(
            Fore.BLUE, Fore.RESET, Fore.GREEN, now.strftime("%H:%M:%S"), Fore.RESET, Fore.BLUE, Fore.RESET, self.END
        )
        if not error_code:
            if not ask:
                print(self.YELLOW + '[!]' + CLOCK + self.BOLD + Fore.YELLOW + text + Fore.RESET + self.END)     

            else:
                return self.YELLOW + '[!]' + CLOCK + self.BOLD + Fore.YELLOW + text + Fore.RESET + self.END

        else:
            if not ask:
                print(CLOCK + f'[{error_code}] ' + text)

            else:
                return CLOCK + f'[{error_code}] ' + text

    
    def error(self, text: str, error_code: str = None):
        now = datetime.now()
        CLOCK = f'{Fore.BLUE}[{Fore.RESET}{Fore.GREEN}{now.strftime("%H:%M:%S")}{Fore.RESET}{Fore.BLUE}]{Fore.RESET}{self.END} '
        if not error_code:
            print(self.RED + '[!]' + CLOCK + self.BOLD + Fore.RED + text + Fore.RESET + self.END)
        elif text2:
            print(self.RED + '[!]' + CLOCK + self.BOLD + Fore.RED + text + Fore.RESET + self.END)
        else:
            print(CLOCK + f'[{error_code}] ' + text)
    
    def info(self, text: str, bold: bool = False):
        now = datetime.now()
        CLOCK = f'{Fore.BLUE}[{Fore.RESET}{Fore.GREEN}{now.strftime("%H:%M:%S")}{Fore.RESET}{Fore.BLUE}]{Fore.RESET}{self.END} '
        if bold:
            print(self.BLUE + '[+]' + CLOCK + self.BOLD + Fore.WHITE + text + Fore.RESET + self.END)
        else:
            print(self.BLUE + '[+]' + CLOCK + Fore.WHITE + text + Fore.RESET + self.END)



import json


class MediaItemsInfoResponse:
    
    def __init__(self, data: dict):

        self.taken_at = None
        self.as_json = data

        if 'taken_at' in data:
            self.taken_at: int = data['taken_at']



import json
from instapv.logger import Logger
from instapv.exceptions import UserNotFoundException

log = Logger()

class UserResponse(object):
    
    def __init__(self, data):
            if type(data) == dict:
                if 'items' in data:
                    data = data['items'][0]['user']
                elif 'user' in data:
                    data = data['user']
                else:
                    data = data
                        
                # Defualt data
                self.as_json = data
                self.biography = data['biography'] if 'biography' in data else None
                self.is_private = data['is_private'] if 'is_private' in data else None
                self.is_business = data['is_business'] if 'is_business' in data else None
                self.account_type = data['account_type'] if 'account_type' in data else None
                self.full_name = data['full_name'] if 'full_name' in data else None
                self.external_lynx_url = data['external_lynx_url'] if 'external_lynx_url' in data else None
                self.external_url = data['external_url'] if 'external_url' in data else None
                self.pk = data['pk'] if 'pk' in data else None
                self.follower_count = data['follower_count'] if 'follower_count' in data else None
                self.following_count = data['following_count'] if 'following_count' in data else None
                self.following_tag_count = data['following_tag_count'] if 'following_tag_count' in data else None
                self.mutual_followers_count = data['mutual_followers_count'] if 'mutual_followers_count' in data else None
                self.geo_media_count = data['geo_media_count'] if 'geo_media_count' in data else None
                self.has_anonymous_profile_picture = data['has_anonymous_profile_picture'] if 'has_anonymous_profile_picture' in data else None
                self.has_biography_translation = data['has_biography_translation'] if 'has_biography_translation' in data else None
                self.has_chaining = data['has_chaining'] if 'has_chaining' in data else None
                self.hd_profile_pic_url = data['hd_profile_pic_url'] if 'hd_profile_pic_url' in data else None
                self.is_favorite = data['is_favorite'] if 'is_favorite' in data else None
                self.is_verified = data['is_verified'] if 'is_verified' in data else None
                self.media_count = data['media_count'] if 'media_count' in data else None
                self.profile_context = data['profile_context'] if 'profile_context' in data else None
                self.profile_pic_url = data['profile_pic_url'] if 'profile_pic_url' in data else None
                self.total_igtv_videos = data['total_igtv_videos'] if 'total_igtv_videos' in data else None
                self.username = data['username'] if 'username' in data else None
                self.usertags_count = data['usertags_count'] if 'usertags_count' in data else None
                self.reel_auto_archive = data['reel_auto_archive'] if 'reel_auto_archive' in data else None
                self.allowed_commenter_type = data['allowed_commenter_type'] if 'allowed_commenter_type' in data else None
                self.professional_conversion_suggested_account_type = data['professional_conversion_suggested_account_type'] if 'professional_conversion_suggested_account_type' in data else None
                self.is_call_to_action_enabled = data['is_call_to_action_enabled'] if 'is_call_to_action_enabled' in data else None
                self.can_see_organic_insights = data['can_see_organic_insights'] if 'can_see_organic_insights' in data else None
                self.show_insights_terms = data['show_insights_terms'] if 'show_insights_terms' in data else None
                self.has_placed_orders = data['has_placed_orders'] if 'has_placed_orders' in data else None
                self.nametag = data['nametag'] if 'nametag' in data else None
                self.is_using_unified_inbox_for_direct = data['is_using_unified_inbox_for_direct'] if 'is_using_unified_inbox_for_direct' in data else None
                self.interop_messaging_user_fbid = data['interop_messaging_user_fbid'] if 'interop_messaging_user_fbid' in data else None
                self.can_see_primary_country_in_settings = data['can_see_primary_country_in_settings'] if 'can_see_primary_country_in_settings' in data else None
                self.allow_contacts_sync = data['allow_contacts_sync'] if 'allow_contacts_sync' in data else None
                self.phone_number = data['phone_number'] if 'phone_number' in data else None
            elif not data:
                raise UserNotFoundException('User not found')


import json

class TwoFactorResponse:

    def __init__(self, data: dict):
        # Return all data as_json
        self.as_json = data
        
        self.username = data['username'] if 'username' in data else None
        self.sms_two_factor_on = data['sms_two_factor_on'] if 'sms_two_factor_on' in data else None
        self.totp_two_factor_on = data['totp_two_factor_on'] if 'totp_two_factor_on' in data else None
        self.obfuscated_phone_number = data['obfuscated_phone_number'] if 'obfuscated_phone_number' in data else None
        self.two_factor_identifier = data['two_factor_identifier'] if 'two_factor_identifier' in data else None
        self.show_messenger_code_option = data['show_messenger_code_option'] if 'show_messenger_code_option' in data else None
        self.show_new_login_screen = data['show_new_login_screen'] if 'show_new_login_screen' in data else None
        self.show_trusted_device_option = data['show_trusted_device_option'] if 'show_trusted_device_option' in data else None


class SentryBlockException(Exception):
    pass

class AccountTypeException(Exception):
    pass

class UserNotFoundException(Exception):
    pass

class InvalidCredentialsException(Exception):
    pass

class RateLimitException(Exception):
    pass

class AccessDeniedException(Exception):
    pass

class ChallengeRequiredException(Exception):
    pass

class LogedOutException(Exception):
    pass

class InternalException(Exception):
    pass


import json
from datetime import datetime

from ..response.live_info import LiveInfoResponse


class Live:

    def __init__(self, bot):
        self.bot = bot

    def create(self, preview_height: int = 1080, preview_width: int = 1920):
        data = {
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid,
            'preview_height': preview_height,
            'preview_width': preview_width,
            'broadcast_type': 'RTMP_SWAP_ENABLED',
            'internal_only': 0
        }
        query = self.bot.request(
            'live/create/', params=data, signed_post=False)
        return LiveInfoResponse(query)

    def start(self, boardcast_id: int):
        data = {
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid
        }
        query = self.bot.request(
            f'live/{boardcast_id}/start/', params=data, signed_post=False)
        return query

    def end(self, boardcast_id: int, copyright_warning: bool = False):
        data = {
            '_uid': self.bot.account_id,
            '_uuid': self.bot.uuid,
            '_csrftoken': self.bot.token,
            'end_after_copyright_warning': copyright_warning
        }
        query = self.bot.request(
            f'live/{boardcast_id}/end_broadcast/', params=data)
        return query

    def save_live(self, boardcast_id: int):
        data = {
            '_uid': self.bot.account_id,
            '_uuid': self.bot.uuid,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request(
            f'live/{boardcast_id}/add_to_post_live/', params=data)
        return query

    def get_post_live_likes(self, boardcast_id: int, starting_offset: int = 0, encoding_tag: str = 'instagram_dash_remuxed'):
        data = {
            'starting_offset': starting_offset,
            'encoding_tag': encoding_tag,
        }
        query = self.bot.request(
            f'live/{boardcast_id}/get_post_live_likes/')
        return query
    
    def get_like_count(self, boardcast_id: int, like_ts: int = 0):
        data = {
            'like_ts': like_ts,
        }
        query = self.bot.request(
            f'live/{boardcast_id}/get_like_count/', params=data)
        return query
    
    def get_comments(self, boardcast_id: int, last_comment_ts: int = 0):
        data = {
            'last_comment_ts': last_comment_ts,
        }
        query = self.bot.request(
            f'live/{boardcast_id}/get_comment/', params=data, signed_post=True)
        return query

    def enable_comments(self, boardcast_id: int):
        data = {
            '_uid': self.bot.account_id,
            '_uuid': self.bot.uuid,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request(
            f'live/{boardcast_id}/unmute_comment/', params=data)
        return query
    
    def disable_comments(self, boardcast_id: int):
        data = {
            '_uid': self.bot.account_id,
            '_uuid': self.bot.uuid,
            '_csrftoken': self.bot.token,
        }
        query = self.bot.request(
            f'live/{boardcast_id}/mute_comment/', params=data)
        return query
        
    def get_live_info(self, boardcast_id: int):
        query = self.bot.request(
            f'live/{boardcast_id}/info/')
        return query


import json
from instapv.response.media_info import MediaInfoResponse
from instapv.response.send_comment import SendCommentInfoResponse
from requests_toolbelt import MultipartEncoder
from time import time

# Fix Python3 Import Error.
try:
    from ImageUtils import getImageSize
except:
    from instapv.utils.ImageUtils import getImageSize


class Media:

    def __init__(self, bot):
        self.bot = bot

    def info(self, media_id: str):
        query = self.bot.request(f'media/{media_id}/info/?')
        return MediaInfoResponse(query)

    def likers(self, media_id: str):
        query = self.bot.request(f'media/{media_id}/likers/?')
        return query

    def comment(self, media_id, comment_text, reply_comment_id = None, module = 'comments_v2', carousel_index = 0, feed_position = 0, feed_bumped = False):
        data = {
            'user_breadcrumb': comment_text,
            'idempotence_token': self.bot.tools.generate_uuid(True),
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'comment_text': comment_text,
            'container_module': module,
            'radio_type': 'wifi-none',
            'device_id': self.bot.device_id,
            'carousel_index': carousel_index,
            'feed_position': feed_position,
            'is_carousel_bumped_post': feed_bumped
        }
        if reply_comment_id != None:
            data.update({
                'replied_to_comment_id': reply_comment_id
            })
        
        query = self.bot.request(f'media/{media_id}/comment/', params=data, signed_post=False)
        return SendCommentInfoResponse(query)

    def get_comment_infos(self, media_ids):
        if isinstance(media_ids, list):
            media_ids = ','.join(media_ids)

        query = self.bot.request(f'media/comment_infos?media_ids={media_ids}')
        return query

    def delete_comment(self, media_id, comment_id):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            
        }
        query = self.bot.request(f'media/{media_id}/comment/{comment_id}/delete/', params=data, signed_post=False)
        return query

    def enable_comments(self, media_id):
        data = {
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid,
        }
        query = self.bot.request(
            f'media/{media_id}/enable_comments/', params=data, signed_post=False)
        return query

    def disable_comments(self, media_id):
        data = {
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid,
        }
        query = self.bot.request(
            f'media/{media_id}/disable_comments/', params=data, signed_post=False)
        return query

    def edit(self, media_id: str, caption_text):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'caption_text': caption_text
        }
        query = self.bot.request(f'media/{media_id}_{self.bot.account_id}/edit_media/', params=data)
        return query

    def delete(self, media_id: str, media_type: str = 'PHOTO'):
        data = {
            'media_type': media_type,
            'igtv_feed_preview': False,
            '_csrftoken': self.bot.token,
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            'media_id': media_id
        }
        return self.bot.request(f'media/{media_id}/delete', params=data)

    def like(self, media_id: str):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'media_id': media_id
        }
        return self.bot.request(f'media/{media_id}/like/', params=data)

    def unlike(self, media_id: str):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'media_id': media_id
        }
        return self.bot.request(f'media/{media_id}/unlike/', params=data)

    def save(self, media_id: str):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'media_id': media_id
        }
        return self.bot.request(f'media/{media_id}/save/', params=data)

    def unsave(self, media_id: str):
        data = {
            '_uuid': self.bot.uuid,
            '_uid': self.bot.account_id,
            '_csrftoken': self.bot.token,
            'media_id': media_id
        }
        return self.bot.request(f'media/{media_id}/unsave/', params=data)

    def get_comments(self, media_id, max_id=''):
        data = {
            'can_support_threading': True,
            'max_id': max_id
        }
        query = self.bot.request(f'media/{media_id}/comments/', params=data)
        return query

    def get_comment_replais(self, media_id, comment_id):
        if not isinstance(comment_id, int):
            raise ValueError('comment_id must be integers.')
        query = self.bot.request(
            f'media/{media_id}/comments/{comment_id}/inline_child_comments/')
        return query


    def upload_photo(self, photo, caption=None, upload_id=None, is_sidecar=None):
        if upload_id is None:
            upload_id = str(int(time() * 1000))
        data = {
            'upload_id': upload_id,
            '_uuid': self.bot.uuid,
            '_csrftoken': self.bot.token,
            'image_compression': '{"lib_name":"jt","lib_version":"1.3.0","quality":"87"}',
            'photo': ('pending_media_%s.jpg' % upload_id, open(photo, 'rb'), 'application/octet-stream', {'Content-Transfer-Encoding': 'binary'})
        }
        if is_sidecar:
            data['is_sidecar'] = '1'
        m = MultipartEncoder(data, boundary=self.bot.uuid)
        self.bot.req.headers.update({
            'X-IG-Capabilities': '3Q4=',
            'X-IG-Connection-Type': 'WIFI',
            'Cookie2': '$Version=1',
            'Accept-Language': 'en-US',
            'Accept-Encoding': 'gzip, deflate',
            'Content-type': m.content_type,
            'Connection': 'close',
            'User-Agent': self.bot.device.build_user_agent()
        })
        response = self.bot.request(
            "upload/photo/", params=m.to_string())
        if response.status_code == 200:
            if self.bot.configure(upload_id, photo, caption):
                self.bot.expose()
        return False

    def configure(self, upload_id, photo, caption=''):
        (w, h) = getImageSize(photo)
        data = json.dumps({
            '_csrftoken': self.bot.token,
            'media_folder': 'Instagram',
            'source_type': 4,
            '_uid': self.bot.account_id,
            '_uuid': self.bot.uuid,
            'caption': caption,
            'upload_id': upload_id,
            'device': self.bot.device.generate_device(),
            'edits': {
                'crop_original_size': [w * 1.0, h * 1.0],
                'crop_center': [0.0, 0.0],
                'crop_zoom': 1.0
            },
            'extra': {
                'source_width': w,
                'source_height': h
            }
        })
        return self.bot.request('media/configure/?', params=data)

    def code_to_media_id(self, short_code: str):
        media_id = 0
        for i in short_code:
            media_id = (media_id*64) + self.bot.config.ALPHABET.index(i)
        return media_id

    def media_id_to_code(self, media_id: int):
        short_code = ''
        while media_id > 0:
            remainder = media_id % 64
            media_id = (media_id-remainder)/64
            short_code = self.bot.config.ALPHABET[remainder] + short_code
        return short_code



import json

class LiveInfoResponse:

    def __init__(self, data: dict):
        if data:
            if data['status'] == 'ok':
                self.broadcast_id = None
                self.upload_url = None

                if 'broadcast_id' in data:
                    self.broadcast_id = data['broadcast_id']
                if 'upload_url' in data:
                    self.upload_url = data['upload_url']



import json
from datetime import datetime

class Business:
    
    def __init__(self, bot):
        self.bot = bot

    def get_insights(self, day = None):
        """
        Get insights.
        :param = day(1-31)
        """
        if not day:
            day = datetime.now().day
        data = {
            'show_promotions_in_landing_page': 'true',
            'first': day
        }
        query = self.bot.request('insights/account_organic_insights/', get_params=data)
        return query
    
    def get_media_insights(self, media_id):
        data = {
            'ig_sig_key_version': self.bot.config.SIG_KEY_VERSION
        }
        query = self.bot.request(f'insights/media_organic_insights/{media_id}/', get_params=data)
        return query
    
    def get_statistics(self):
        _params = {
            'locale': self.bot.config.USER_AGENT_LOCALE,
            'vc_policy': 'insights_policy',
            'surface': 'account',
        }

        data = {
            'access_token': 'undefined',
            'fb_api_caller_class': 'RelayModern',
            'variables': json.dumps({
                'IgInsightsGridMediaImage_SIZE': 240,
                'timezone': 'Atlantic/Canary',
                'activityTab': True,
                'audienceTab': True,
                'contentTab': True,
                'query_params': json.dumps({
                    'access_token': '',
                    'id': self.bot.account_id
                }),
            }),
            'doc_id': '1926322010754880'
        }

        query = self.bot.request('ads/graphql/', get_params=_params, params=data, signed_post=False)
        return query